@@ -216,10 +216,10 @@ class Application(Generic[_AppResult]):
         max_render_postpone_time: Union[float, int, None] = 0.01,
         refresh_interval: Optional[float] = None,
         terminal_size_polling_interval: Optional[float] = 0.5,
-        on_reset: Optional[ApplicationEventHandler] = None,
-        on_invalidate: Optional[ApplicationEventHandler] = None,
-        before_render: Optional[ApplicationEventHandler] = None,
-        after_render: Optional[ApplicationEventHandler] = None,
+        on_reset: Optional["ApplicationEventHandler[_AppResult]"] = None,
+        on_invalidate: Optional["ApplicationEventHandler[_AppResult]"] = None,
+        before_render: Optional["ApplicationEventHandler[_AppResult]"] = None,
+        after_render: Optional["ApplicationEventHandler[_AppResult]"] = None,
         # I/O.
         input: Optional[Input] = None,
         output: Optional[Output] = None,
@@ -504,7 +504,7 @@ class Application(Generic[_AppResult]):
 
     @property
     def invalidated(self) -> bool:
-        " True when a redraw operation has been scheduled. "
+        "True when a redraw operation has been scheduled."
         return self._invalidated
 
     def _redraw(self, render_as_done: bool = False) -> None:
@@ -556,9 +556,9 @@ class Application(Generic[_AppResult]):
         Start a while/true loop in the background for automatic invalidation of
         the UI.
         """
-        if self.refresh_interval not in (None, 0):
+        if self.refresh_interval is not None and self.refresh_interval != 0:
 
-            async def auto_refresh(refresh_interval) -> None:
+            async def auto_refresh(refresh_interval: float) -> None:
                 while True:
                     await sleep(refresh_interval)
                     self.invalidate()
@@ -651,7 +651,7 @@ class Application(Generic[_AppResult]):
         assert not self._is_running, "Application is already running."
 
         async def _run_async() -> _AppResult:
-            " Coroutine. "
+            "Coroutine."
             loop = get_event_loop()
             f = loop.create_future()
             self.future = f  # XXX: make sure to set this before calling '_redraw'.
@@ -769,7 +769,7 @@ class Application(Generic[_AppResult]):
                         # Store unprocessed input as typeahead for next time.
                         store_typeahead(self.input, self.key_processor.empty_queue())
 
-                return result
+                return cast(_AppResult, result)
 
         async def _run_async2() -> _AppResult:
             self._is_running = True
@@ -787,7 +787,7 @@ class Application(Generic[_AppResult]):
                 loop.set_exception_handler(self._handle_exception)
 
             try:
-                with set_app(self):
+                with set_app(self), self._enable_breakpointhook():
                     try:
                         result = await _run_async()
                     finally:
@@ -839,7 +839,11 @@ class Application(Generic[_AppResult]):
             won't use the current event loop (asyncio does not support nested
             event loops). A new event loop will be created in this background
             thread, and that loop will also be closed when the background
-            thread terminates. This is used for instance in ptpython.
+            thread terminates. When this is used, it's especially important to
+            make sure that all asyncio background tasks are managed through
+            `get_appp().create_background_task()`, so that unfinished tasks are
+            properly cancelled before the event loop is closed. This is used
+            for instance in ptpython.
         """
         if in_thread:
             result: _AppResult
@@ -914,11 +918,63 @@ class Application(Generic[_AppResult]):
 
         ensure_future(in_term())
 
+    @contextmanager
+    def _enable_breakpointhook(self) -> Generator[None, None, None]:
+        """
+        Install our custom breakpointhook for the duration of this context
+        manager. (We will only install the hook if no other custom hook was
+        set.)
+        """
+        if sys.version_info >= (3, 7) and sys.breakpointhook == sys.__breakpointhook__:
+            sys.breakpointhook = self._breakpointhook
+
+            try:
+                yield
+            finally:
+                sys.breakpointhook = sys.__breakpointhook__
+        else:
+            yield
+
+    def _breakpointhook(self, *a: object, **kw: object) -> None:
+        """
+        Breakpointhook which uses PDB, but ensures that the application is
+        hidden and input echoing is restored during each debugger dispatch.
+        """
+        app = self
+        # Inline import on purpose. We don't want to import pdb, if not needed.
+        import pdb
+        from types import FrameType
+
+        TraceDispatch = Callable[[FrameType, str, Any], Any]
+
+        class CustomPdb(pdb.Pdb):
+            def trace_dispatch(
+                self, frame: FrameType, event: str, arg: Any
+            ) -> TraceDispatch:
+                # Hide application.
+                app.renderer.erase()
+
+                # Detach input and dispatch to debugger.
+                with app.input.detach():
+                    with app.input.cooked_mode():
+                        return super().trace_dispatch(frame, event, arg)
+
+                # Note: we don't render the application again here, because
+                # there's a good chance that there's a breakpoint on the next
+                # line. This paint/erase cycle would move the PDB prompt back
+                # to the middle of the screen.
+
+        frame = sys._getframe().f_back
+        CustomPdb(stdout=sys.__stdout__).set_trace(frame)
+
     def create_background_task(
         self, coroutine: Awaitable[None]
     ) -> "asyncio.Task[None]":
         """
-        Start a background task (coroutine) for the running application.
+        Start a background task (coroutine) for the running application. When
+        the `Application` terminates, unfinished background tasks will be
+        cancelled.
+
         If asyncio had nurseries like Trio, we would create a nursery in
         `Application.run_async`, and run the given coroutine in that nursery.
 
@@ -985,17 +1041,17 @@ class Application(Generic[_AppResult]):
 
     @overload
     def exit(self) -> None:
-        " Exit without arguments. "
+        "Exit without arguments."
 
     @overload
     def exit(self, *, result: _AppResult, style: str = "") -> None:
-        " Exit with `_AppResult`. "
+        "Exit with `_AppResult`."
 
     @overload
     def exit(
         self, *, exception: Union[BaseException, Type[BaseException]], style: str = ""
     ) -> None:
-        " Exit with exception. "
+        "Exit with exception."
 
     def exit(
         self,
@@ -1134,7 +1190,7 @@ class Application(Generic[_AppResult]):
 
     @property
     def is_running(self) -> bool:
-        " `True` when the application is currently active/running. "
+        "`True` when the application is currently active/running."
         return self._is_running
 
     @property
@@ -1273,7 +1329,7 @@ async def _do_wait_for_enter(wait_text: AnyFormattedText) -> None:
 
     @key_bindings.add(Keys.Any)
     def _ignore(event: E) -> None:
-        " Disallow typing. "
+        "Disallow typing."
         pass
 
     session: PromptSession[None] = PromptSession(
