@@ -186,7 +186,7 @@ class UIContent:
         self._line_heights_cache: Dict[Hashable, int] = {}
 
     def __getitem__(self, lineno: int) -> StyleAndTextTuples:
-        " Make it iterable (iterate line by line). "
+        "Make it iterable (iterate line by line)."
         if lineno < self.line_count:
             return self.get_line(lineno)
         else:
@@ -375,7 +375,9 @@ class FormattedTextControl(UIControl):
         wrap_lines: bool,
         get_line_prefix: Optional[GetLinePrefixCallable],
     ) -> Optional[int]:
-
+        """
+        Return the preferred height for this control.
+        """
         content = self.create_content(width, None)
         if wrap_lines:
             height = 0
@@ -463,7 +465,7 @@ class FormattedTextControl(UIControl):
                 count = 0
                 for item in fragments:
                     count += len(item[1])
-                    if count >= xpos:
+                    if count > xpos:
                         if len(item) >= 3:
                             # Handler found. Call it.
                             # (Handler can return NotImplemented, so return
@@ -545,7 +547,7 @@ class BufferControl(UIControl):
         search_buffer_control: Union[
             None, "SearchBufferControl", Callable[[], "SearchBufferControl"]
         ] = None,
-        menu_position: Optional[Callable] = None,
+        menu_position: Optional[Callable[[], Optional[int]]] = None,
         focus_on_click: FilterOrBool = False,
         key_bindings: Optional["KeyBindingsBase"] = None,
     ):
@@ -693,7 +695,7 @@ class BufferControl(UIControl):
         merged_processor = merge_processors(input_processors)
 
         def transform(lineno: int, fragments: StyleAndTextTuples) -> _ProcessedLine:
-            " Transform the fragments for a given line number. "
+            "Transform the fragments for a given line number."
             # Get cursor position at this line.
             def source_to_display(i: int) -> int:
                 """X position from the buffer to the x position in the
@@ -782,11 +784,11 @@ class BufferControl(UIControl):
         self._last_get_processed_line = get_processed_line
 
         def translate_rowcol(row: int, col: int) -> Point:
-            " Return the content column for this coordinate. "
+            "Return the content column for this coordinate."
             return Point(x=get_processed_line(row).source_to_display(col), y=row)
 
         def get_line(i: int) -> StyleAndTextTuples:
-            " Return the fragments for a given line number. "
+            "Return the fragments for a given line number."
             fragments = get_processed_line(i).fragments
 
             # Add a space at the end, because that is a possible cursor
