@@ -8,20 +8,24 @@ http://pygments.org/
 """
 import array
 import errno
+import io
 import os
 import sys
+from contextlib import contextmanager
 from typing import (
     IO,
     Callable,
     Dict,
     Hashable,
     Iterable,
+    Iterator,
     List,
     Optional,
     Sequence,
     Set,
     TextIO,
     Tuple,
+    cast,
 )
 
 from prompt_toolkit.data_structures import Size
@@ -266,7 +270,8 @@ _256_colors = _256ColorCache()
 class _EscapeCodeCache(Dict[Attrs, str]):
     """
     Cache for VT100 escape codes. It maps
-    (fgcolor, bgcolor, bold, underline, reverse) tuples to VT100 escape sequences.
+    (fgcolor, bgcolor, bold, underline, strike, reverse) tuples to VT100
+    escape sequences.
 
     :param true_color: When True, use 24bit colors instead of 256 colors.
     """
@@ -275,7 +280,17 @@ class _EscapeCodeCache(Dict[Attrs, str]):
         self.color_depth = color_depth
 
     def __missing__(self, attrs: Attrs) -> str:
-        fgcolor, bgcolor, bold, underline, italic, blink, reverse, hidden = attrs
+        (
+            fgcolor,
+            bgcolor,
+            bold,
+            underline,
+            strike,
+            italic,
+            blink,
+            reverse,
+            hidden,
+        ) = attrs
         parts: List[str] = []
 
         parts.extend(self._colors_to_code(fgcolor or "", bgcolor or ""))
@@ -292,6 +307,8 @@ class _EscapeCodeCache(Dict[Attrs, str]):
             parts.append("7")
         if hidden:
             parts.append("8")
+        if strike:
+            parts.append("9")
 
         if parts:
             result = "\x1b[0;" + ";".join(parts) + "m"
@@ -302,7 +319,7 @@ class _EscapeCodeCache(Dict[Attrs, str]):
         return result
 
     def _color_name_to_rgb(self, color: str) -> Tuple[int, int, int]:
-        " Turn 'ffffff', into (0xff, 0xff, 0xff). "
+        "Turn 'ffffff', into (0xff, 0xff, 0xff)."
         try:
             rgb = int(color, 16)
         except ValueError:
@@ -372,30 +389,14 @@ class _EscapeCodeCache(Dict[Attrs, str]):
 
 
 def _get_size(fileno: int) -> Tuple[int, int]:
-    # Thanks to fabric (fabfile.org), and
-    # http://sqizit.bartletts.id.au/2011/02/14/pseudo-terminals-in-python/
     """
     Get the size of this pseudo terminal.
 
     :param fileno: stdout.fileno()
     :returns: A (rows, cols) tuple.
     """
-    # Inline imports, because these modules are not available on Windows.
-    # (This file is used by ConEmuOutput, which is used on Windows.)
-    import fcntl
-    import termios
-
-    # Buffer for the C call
-    buf = array.array("h", [0, 0, 0, 0])
-
-    # Do TIOCGWINSZ (Get)
-    # Note: We should not pass 'True' as a fourth parameter to 'ioctl'. (True
-    #       is the default.) This causes segmentation faults on some systems.
-    #       See: https://github.com/jonathanslenders/python-prompt-toolkit/pull/364
-    fcntl.ioctl(fileno, termios.TIOCGWINSZ, buf)  # type: ignore
-
-    # Return rows, cols
-    return buf[0], buf[1]
+    size = os.get_terminal_size(fileno)
+    return size.lines, size.columns
 
 
 class Vt100_Output(Output):
@@ -418,6 +419,7 @@ class Vt100_Output(Output):
         term: Optional[str] = None,
         write_binary: bool = True,
         default_color_depth: Optional[ColorDepth] = None,
+        enable_bell: bool = True,
     ) -> None:
 
         assert all(hasattr(stdout, a) for a in ("write", "flush"))
@@ -426,11 +428,12 @@ class Vt100_Output(Output):
             assert hasattr(stdout, "encoding")
 
         self._buffer: List[str] = []
-        self.stdout = stdout
+        self.stdout: TextIO = stdout
         self.write_binary = write_binary
         self.default_color_depth = default_color_depth
         self._get_size = get_size
         self.term = term
+        self.enable_bell = enable_bell
 
         # Cache for escape codes.
         self._escape_code_caches: Dict[ColorDepth, _EscapeCodeCache] = {
@@ -446,22 +449,28 @@ class Vt100_Output(Output):
         stdout: TextIO,
         term: Optional[str] = None,
         default_color_depth: Optional[ColorDepth] = None,
+        enable_bell: bool = True,
     ) -> "Vt100_Output":
         """
         Create an Output class from a pseudo terminal.
         (This will take the dimensions by reading the pseudo
         terminal attributes.)
         """
+        fd: Optional[int]
         # Normally, this requires a real TTY device, but people instantiate
         # this class often during unit tests as well. For convenience, we print
         # an error message, use standard dimensions, and go on.
-        fd = stdout.fileno()
+        try:
+            fd = stdout.fileno()
+        except io.UnsupportedOperation:
+            fd = None
 
-        if not stdout.isatty() and fd not in cls._fds_not_a_terminal:
+        if not stdout.isatty() and (fd is None or fd not in cls._fds_not_a_terminal):
             msg = "Warning: Output is not a terminal (fd=%r).\n"
             sys.stderr.write(msg % fd)
             sys.stderr.flush()
-            cls._fds_not_a_terminal.add(fd)
+            if fd is not None:
+                cls._fds_not_a_terminal.add(fd)
 
         def get_size() -> Size:
             # If terminal (incorrectly) reports its size as 0, pick a
@@ -479,17 +488,23 @@ class Vt100_Output(Output):
                 pass
             return Size(rows=rows or 24, columns=columns or 80)
 
-        return cls(stdout, get_size, term=term, default_color_depth=default_color_depth)
+        return cls(
+            stdout,
+            get_size,
+            term=term,
+            default_color_depth=default_color_depth,
+            enable_bell=enable_bell,
+        )
 
     def get_size(self) -> Size:
         return self._get_size()
 
     def fileno(self) -> int:
-        " Return file descriptor. "
+        "Return file descriptor."
         return self.stdout.fileno()
 
     def encoding(self) -> str:
-        " Return encoding used for stdout. "
+        "Return encoding used for stdout."
         return self.stdout.encoding
 
     def write_raw(self, data: str) -> None:
@@ -590,6 +605,14 @@ class Vt100_Output(Output):
     def disable_bracketed_paste(self) -> None:
         self.write_raw("\x1b[?2004l")
 
+    def reset_cursor_key_mode(self) -> None:
+        """
+        For vt100 only.
+        Put the terminal in cursor mode (instead of application mode).
+        """
+        # Put the terminal in cursor mode. (Instead of application mode.)
+        self.write_raw("\x1b[?1l")
+
     def cursor_goto(self, row: int = 0, column: int = 0) -> None:
         """
         Move cursor position.
@@ -644,24 +667,32 @@ class Vt100_Output(Output):
             return
 
         data = "".join(self._buffer)
+        self._buffer = []
 
         try:
-            # (We try to encode ourself, because that way we can replace
-            # characters that don't exist in the character set, avoiding
-            # UnicodeEncodeError crashes. E.g. u'\xb7' does not appear in 'ascii'.)
-            # My Arch Linux installation of july 2015 reported 'ANSI_X3.4-1968'
-            # for sys.stdout.encoding in xterm.
-            out: IO
-            if self.write_binary:
-                if hasattr(self.stdout, "buffer"):
-                    out = self.stdout.buffer  # Py3.
+            # Ensure that `self.stdout` is made blocking when writing into it.
+            # Otherwise, when uvloop is activated (which makes stdout
+            # non-blocking), and we write big amounts of text, then we get a
+            # `BlockingIOError` here.
+            with blocking_io(self.stdout):
+                # (We try to encode ourself, because that way we can replace
+                # characters that don't exist in the character set, avoiding
+                # UnicodeEncodeError crashes. E.g. u'\xb7' does not appear in 'ascii'.)
+                # My Arch Linux installation of july 2015 reported 'ANSI_X3.4-1968'
+                # for sys.stdout.encoding in xterm.
+                out: IO[bytes]
+                if self.write_binary:
+                    if hasattr(self.stdout, "buffer"):
+                        out = self.stdout.buffer
+                    else:
+                        # IO[bytes] was given to begin with.
+                        # (Used in the unit tests, for instance.)
+                        out = cast(IO[bytes], self.stdout)
+                    out.write(data.encode(self.stdout.encoding or "utf-8", "replace"))
                 else:
-                    out = self.stdout
-                out.write(data.encode(self.stdout.encoding or "utf-8", "replace"))
-            else:
-                self.stdout.write(data)
+                    self.stdout.write(data)
 
-            self.stdout.flush()
+                self.stdout.flush()
         except IOError as e:
             if e.args and e.args[0] == errno.EINTR:
                 # Interrupted system call. Can happen in case of a window
@@ -679,8 +710,6 @@ class Vt100_Output(Output):
             else:
                 raise
 
-        self._buffer = []
-
     def ask_for_cpr(self) -> None:
         """
         Asks for a cursor position report (CPR).
@@ -703,9 +732,10 @@ class Vt100_Output(Output):
             return False  # ValueError: I/O operation on closed file
 
     def bell(self) -> None:
-        " Sound bell. "
-        self.write_raw("\a")
-        self.flush()
+        "Sound bell."
+        if self.enable_bell:
+            self.write_raw("\a")
+            self.flush()
 
     def get_default_color_depth(self) -> ColorDepth:
         """
@@ -730,3 +760,37 @@ class Vt100_Output(Output):
             return ColorDepth.DEPTH_4_BIT
 
         return ColorDepth.DEFAULT
+
+
+@contextmanager
+def blocking_io(io: IO[str]) -> Iterator[None]:
+    """
+    Ensure that the FD for `io` is set to blocking in here.
+    """
+    if sys.platform == "win32":
+        # On Windows, the `os` module doesn't have a `get/set_blocking`
+        # function.
+        yield
+        return
+
+    try:
+        fd = io.fileno()
+        blocking = os.get_blocking(fd)
+    except:  # noqa
+        # Failed somewhere.
+        # `get_blocking` can raise `OSError`.
+        # The io object can raise `AttributeError` when no `fileno()` method is
+        # present if we're not a real file object.
+        blocking = True  # Assume we're good, and don't do anything.
+
+    try:
+        # Make blocking if we weren't blocking yet.
+        if not blocking:
+            os.set_blocking(fd, True)
+
+        yield
+
+    finally:
+        # Restore original blocking mode.
+        if not blocking:
+            os.set_blocking(fd, blocking)
